-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity digit_counter_10 is
  port(
    clk_i          : in std_logic;
    rst_i          : in std_logic;
    tick_i         : in boolean;
    count_oq       : out std_logic_vector(3 downto 0);
    overflow_oq : out boolean);
end digit_counter_10;

architecture behave of digit_counter_10 is
  signal count_ff_q : unsigned(3 downto 0);
  signal overflow_ff_q : boolean;
begin  -- behave


  process(clk_i)
  begin
    if rising_edge(clk_i) then
      overflow_ff_q     <= false;
      if rst_i = '1' then
        count_ff_q       <= "0000";
      elsif tick_i then
        if count_ff_q = "1001" then
          overflow_ff_q <= true;
        count_ff_q       <= "0000";
      else
        count_ff_q       <= count_ff_q + 1;
      end if;
    end if;
  end if;
end process;
count_oq                 <= std_logic_vector(count_ff_q);
overflow_oq              <= overflow_ff_q;
end behave;  

-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity digit_counter_6 is
  port(
    clk_i          : in std_logic;
    rst_i          : in std_logic;
    tick_i         : in boolean;
    count_oq       : out std_logic_vector(2 downto 0);
    overflow_oq : out boolean);
end digit_counter_6;

architecture behave of digit_counter_6 is
  signal count_ff_q : unsigned(2 downto 0);
  signal overflow_ff_q : boolean;
begin  -- behave
  process(clk_i)
  begin
    if rising_edge(clk_i) then
      overflow_ff_q     <= false;
      if rst_i = '1' then
        count_ff_q       <= "000";
      elsif tick_i then
        if count_ff_q = "110" then
          overflow_ff_q <= true;
        count_ff_q       <= "000";
      else
        count_ff_q       <= count_ff_q + 1;
      end if;
    end if;
  end if;
end process;
count_oq                 <= std_logic_vector(count_ff_q);
overflow_oq              <= overflow_ff_q;
end behave;  

-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity digit_counter_2 is
  port(
    clk_i          : in std_logic;
    rst_i          : in std_logic;
    tick_i         : in boolean;
    count_oq       : out std_logic_vector(1 downto 0);
    overflow_oq : out boolean);
end digit_counter_2;

architecture behave of digit_counter_2 is
  signal count_ff_q : unsigned(1 downto 0);
  signal overflow_ff_q : boolean;
begin  -- behave


  process(clk_i)
  begin
    if rising_edge(clk_i) then
      overflow_ff_q     <= false;
      if rst_i = '1' then
        count_ff_q       <= "00";
      elsif tick_i then
        if count_ff_q = "10" then
          overflow_ff_q <= true;
        count_ff_q       <= "00";
      else
        count_ff_q       <= count_ff_q + 1;
      end if;
    end if;
  end if;
end process;
count_oq                 <= std_logic_vector(count_ff_q);
overflow_oq              <= overflow_ff_q;
end behave;  
-------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------
-- ----------------------------------------------------------------------------
-- 
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity cpld_lcd is

  port (
    clk_i           : in    std_logic;
    rst_i           : in    std_logic;
    --lcd interface
    lcd_data_o      : inout std_logic_vector(7 downto 0);
    lcd_rs_o        : out   std_logic;
    lcd_rw_o        : out   std_logic;
    lcd_e_o         : out   std_logic;  --7/11 pins
    --rom interface
    rom_adr_o       : out   std_logic_vector(10 downto 0);
    rom_data_io     : inout std_logic_vector(7 downto 0);
    rom_cen_o       : out   std_logic;
    rom_oen_o       : out   std_logic;
    rom_wen_o       : out   std_logic;
    rom_rdy_nbusy_i : in    std_logic
    );

end cpld_lcd;

architecture behave of cpld_lcd is
  signal sec_tick_q, min_tick_q, hrs_tick_q, day_tick_q : boolean;
  signal sec_low_co, min_low_co, hrs_low_co : boolean;

  signal prescaler_q     : unsigned(14 downto 0);
  signal all_ticks_muxed : boolean;

  signal digit_sl : std_logic_vector(7 downto 0);
  signal digit_sh : std_logic_vector(7 downto 0);
  signal digit_ml : std_logic_vector(7 downto 0);
  signal digit_mh : std_logic_vector(7 downto 0);
  signal digit_hl : std_logic_vector(7 downto 0);
  signal digit_hh : std_logic_vector(7 downto 0);

  signal bit_del_q : unsigned(0 downto 0);
  signal ena_tick : boolean;
  signal digit_number_q : unsigned(2 downto 0);

  signal next_digit, all_digits_done : boolean;
  type STATE_TYPE is (IDLE,ENA_LOW,ENA_HIGH);
  signal state_q : STATE_TYPE;

  
begin  -- behave

  sec_low: entity work.digit_counter_10
    port map (
        clk_i       => clk_i,
        rst_i       => rst_i,
        tick_i      => sec_tick_q,
        count_oq    => digit_sl(3 downto 0),
        overflow_oq => sec_low_co);

 sec_high: entity work.digit_counter_6
    port map (
        clk_i       => clk_i,
        rst_i       => rst_i,
        tick_i      => sec_low_co,
        count_oq    => digit_sh(2 downto 0),
        overflow_oq => min_tick_q);

  min_low: entity work.digit_counter_10
    port map (
        clk_i       => clk_i,
        rst_i       => rst_i,
        tick_i      => min_tick_q,
        count_oq    => digit_ml(3 downto 0),
        overflow_oq => min_low_co);

  min_high: entity work.digit_counter_6
    port map (
        clk_i       => clk_i,
        rst_i       => rst_i,
        tick_i      => min_low_co,
        count_oq    => digit_mh(2 downto 0),
        overflow_oq => hrs_tick_q);

  hrs_low: entity work.digit_counter_10
    port map (
        clk_i       => clk_i,
        rst_i       => rst_i,
        tick_i      => hrs_tick_q,
        count_oq    => digit_hl(3 downto 0),
        overflow_oq => hrs_low_co);

  hrs_high: entity work.digit_counter_2
    port map (
        clk_i       => clk_i,
        rst_i       => rst_i,
        tick_i      => hrs_low_co,
        count_oq    => digit_hh(1 downto 0),
        overflow_oq => day_tick_q);

--fill bits TO LCD character code

  digit_sl(7 downto 4) <= "0011";
  digit_ml(7 downto 4) <= "0011";
  digit_hl(7 downto 4) <= "0011";
  digit_sh(7 downto 3) <= "00110";
  digit_mh(7 downto 3) <= "00110";
  digit_hh(7 downto 2) <= "001100";

  --opposite display order ?
  with digit_number_q select
    lcd_data_o(3 downto 0) <=  digit_sl(3 downto 0) when "000",
                   digit_sh(3 downto 0) when "001",
                   digit_ml(3 downto 0) when "011",
                   digit_mh(3 downto 0) when "100",
                   digit_hl(3 downto 0) when "110",
                   digit_hh(3 downto 0) when "111",
                   "1010" when others;  -- code for ":"

  lcd_data_o(7 downto 4) <= "0011";
  
  --digit_number selects digit to display, is counter
  process(clk_i)
  begin
    if rising_edge(clk_i) then
      if rst_i = '1' then
        digit_number_q <= "000";
      elsif next_digit then
        if digit_number_q = "111" then
          digit_number_q  <= "000" ;
       else
         digit_number_q <= digit_number_q + 1;
      end if;
    end if;
  end if;
  end process;
  all_digits_done <= digit_number_q = "111";

  --FSM displaying data
  process(clk_i)
   begin
     if rising_edge(clk_i) then
       next_digit <= false;
       lcd_e_o <= '0';
       if rst_i = '1' then
         state_q <= IDLE;
       else
         case state_q is
           when IDLE =>
             lcd_e_o <= '0';
             if sec_tick_q then
               state_q <= ENA_HIGH;
             end if;

           when ENA_HIGH => 
            lcd_e_o <= '1';
             if ena_tick then
               state_q <= ENA_LOW;               
             end if;               

           when ENA_LOW => 
               lcd_e_o <= '0' ;
               if ena_tick then
                 if all_digits_done then
                   state_q <= IDLE;
                 else
                   next_digit <= true;
                   state_q <= ENA_HIGH;
                 end if;   
               end if;

           when others => null;
         end case;
       end if;
     end if;
   end process;

  lcd_rs_o <= '0';
  lcd_rw_o <=  '0';

--prescaler (based on 32.xyz kHz Uhrenquartz)
--period 30.5 us
  process(clk_i)
  begin
    if rising_edge(clk_i) then
      sec_tick_q    <= false;
      bit_del_q(0) <= prescaler_q(0);
      if rst_i = '1' then
        prescaler_q <= (others => '0');
      elsif prescaler_q = "111111111111111" then
        sec_tick_q  <= true;
        prescaler_q <= "000000000000000";
      else
        prescaler_q <= prescaler_q + 1;
      end if;
    end if;
  end process;
  ena_tick <=  prescaler_q(0) /= bit_del_q(0);
  

--dummy to prevent dead logic elimination

--   all_ticks_muxed <= sec_tick_q when lcd_data_o(1 downto 0) = "00" else
--                      min_tick_q when lcd_data_o(1 downto 0) = "01" else
--                      hrs_tick_q when lcd_data_o(1 downto 0) = "10" else
--                      day_tick_q when lcd_data_o(1 downto 0) = "11" else
--                      false;
--   rom_data_io(0)  <= '1'        when all_ticks_muxed               else '0';

end behave;

