--when using evalboard with 1.8 MHz clock
--devide high clock down to 32 kHz, so no cpld change is needed
--at final product
-- CPLD1: Coolrunner -> clock divider
-- cpld2: xc9500     -> lcd clock


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity clk_div is
  port(
    clk_i     : in  std_logic;
    rst_i     : in  std_logic;
    clk_32k_o : out std_logic);
end entity clk_div;

architecture behave of clk_div is

  signal clk_32k_q : std_logic;
  subtype COUNT_TYPE is integer range 27 downto 0;
  signal count_q   : COUNT_TYPE;

begin  -- behave
--clockdivider from 1.8432 MHz to 32.768 kHz Uhrenquartz
--shall: period from 0.542 us to 30.5 us
--shall: ratio 56.25
--is:    ratio 56 -> period 30.384 us
  clk_32k_o <= clk_32k_q;

  process(clk_i, rst_i)
  begin
    if rst_i = '1' then
      count_q   <= COUNT_TYPE'low;
      clk_32k_q     <= '0';
    elsif rising_edge(clk_i) then
      if count_q = COUNT_TYPE'high then
        clk_32k_q   <= not clk_32k_q;
        count_q <= COUNT_TYPE'low;
      else
        count_q <= count_q + 1;
      end if;
    end if;
  end process;

end architecture behave;
